package main

import (
	"bufio";
	"os";
	"fmt";
	"strconv";
	"regexp";
)

const BOARD_LENGTH = 20

const PLAYER1 = "*";
const PLAYER2 = "@";

type Board [BOARD_LENGTH*BOARD_LENGTH]int

func (b *Board) Display() {
	fmt.Print("   ");
	for i := 0; i < BOARD_LENGTH; i++ {
		fmt.Printf("%3d", i);
	}
	fmt.Println();

	for i, n := range *b {
		x := i % BOARD_LENGTH;
		if x == 0 {
			fmt.Printf("%3d", i / BOARD_LENGTH);
		}

		switch n {
		case 1:  fmt.Printf("  %s", PLAYER1);
		case 2:  fmt.Printf("  %s", PLAYER2);
		default: fmt.Print("  -");
		}
		if x == BOARD_LENGTH - 1 {
			fmt.Println();
		}
	}
}

func inRange(n int) bool {
	return 0 <= n && n < BOARD_LENGTH;
}

func (b *Board) Put(x, y, n int) bool {
	if (!inRange(x) || !inRange(y)) {
		fmt.Println("out of index");
		return false;
	}
	index := x + y * BOARD_LENGTH;
	if b[index] != 0 {
		fmt.Println("already put stone");
		return false;
	}
	b[index] = n;
	return true;
}

func (b *Board) IsWinner(putx, puty, putn int) bool {
	for i := 0; i < 4; i++ {
		var xx, yy int;
		switch i {
		case 0: xx, yy = -1, -1;
		case 1: xx, yy = -1,  0;
		case 2: xx, yy =  0, -1;
		case 3: xx, yy = -1,  1;
		}

		x, y := putx, puty;
		five := 1;
		d := 1;
		for {
			x += (xx * d);
			y += (yy * d);
			if inRange(x) && inRange(y) &&
				b[x + y * BOARD_LENGTH] == putn {
				if five++; five >= 5 {
					return true;
				}
				continue;
			}

			if d == -1 {
				break;
			}
			x, y = putx, puty;
			d = -1;
		}
	}
	return false;
}

func main() {
	var stones = []string{" ", PLAYER1, PLAYER2};
	turn := 1;
	board := new(Board);
	board.Display();


	fmt.Printf("player%d's(%s) turn\n", turn, stones[turn]);

	reXY := regexp.MustCompile(`^ *([0-9]+) +([0-9]+) *\n$`);

	in := bufio.NewReader(os.Stdin);
	for {
		lineBuf, err := in.ReadString('\n');
		if err != nil {
			fmt.Fprintln(os.Stderr, "ReadLine err:", err);
			os.Exit(2);
		}
		line := string(lineBuf);
		if line == "exit\n" {
			os.Exit(0);
		}
		params := reXY.MatchStrings(line);

		if len(params) > 0 {
			x, _ := strconv.Atoi(params[1]);
			y, _ := strconv.Atoi(params[2]);
			if board.Put(x, y, turn) {
				board.Display();
				if board.IsWinner(x, y, turn) {
					fmt.Printf("player%d win\n", turn);
					os.Exit(0);
				}

				if turn == 1 { turn = 2; }
				else { turn = 1; }
				fmt.Printf("player%d's(%s) turn\n", turn, stones[turn]);
			}
		}
    }
}
